/*

montecarlo.c

(C) 1997 lars brinkhoff.

1997-12-29: Tomas' birthday.
1997-12-29: A nice little animation produced.
*/

#define X_MIN -100.0
#define X_MAX 100.0
#define Y_MIN -100.0
#define Y_MAX 100.0

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

#include "ray.h"
#include "objects.h"

struct Options
{
  int N;
  int width;
  int height;
  int rays_per_pixel;
};

static Real trace (Ray, int);
static inline void find_first_intersection (Ray, Object **, Real *);
static void parse_options (int, char *[], struct Options *);

Object *objects[100];

extern char *optarg;

Object *light, *rings;

static inline Real render_pixel (Ray ray, Real x, Real y,
				 Real a, Real c, int n)
{
  Real I = 0.0;
  Real b, d, u, v;
  int i, j;
  
  b = a / n;
  d = c / n;
  
  for (i = 0, u = 0.0; i < n; i++, u += b)
    for (j = 0, v = 0.0; j < n; j++, v += d)
      {
	ray.d.x = x - ray.p.x + u;
	ray.d.y = y - ray.p.y + v;
	ray.d.z = 0.0 - ray.p.z;
	vector_normalize (ray.d);
	
	I += trace (ray, 1);
      }
  
  return I / (n * n);
}

void
render (int view_width, int view_height, int rays_per_pixel)
{
  int X, Y, J;
  Real x, y, delta_x, delta_y, I;
  Ray ray;
  
  delta_x = (X_MAX - X_MIN) / view_width;
  delta_y = (Y_MAX - Y_MIN) / view_height;
  
  ray.p = (Vector) { 0.0, 0.0, -150.0 };

  for (Y = 0, y = Y_MAX; Y < view_height; Y++, y -= delta_y)
    for (X = 0, x = X_MIN; X < view_width; X++, x += delta_x)
      {
#if 1
	I = render_pixel (ray, x, y, delta_x, delta_y, rays_per_pixel);
#else
	ray.d.x = x - ray.p.x;
	ray.d.y = y - ray.p.y;
	ray.d.z = 0.0 - ray.p.z;
        vector_normalize (ray.d);
	
	I = trace (ray, 1);
#endif
	I = I < 0.0 ? 0.0 : I;
	I = I > 1.0 ? 1.0 : I;
	J = (int)(255.0 * I + .5);
	printf ("%c%c%c", J, J, J);
      }
}

int
main (int argc, char *argv[])
{
  struct Options options;
  int i;

  parse_options (argc, argv, &options);

  objects[0] = create_sphere (vec (0.0, 30.0, 0.0), 40.0);
  
  for (i = 1; i < 11; i++)
    {
      double x, z;
      
      x = 80 * cos (2 * M_PI * i / 10.0 + 2 * M_PI / 10.0 / 16.0 * options.N);
      z = 80 * sin (2 * M_PI * i / 10.0 + 2 * M_PI / 10.0 / 16.0 * options.N);
      objects[i] = create_sphere (vec (x, -20.0, z), 15.0);
    }

  light = create_sphere (vec (60.0, 60.0, -30.0), 15.0);
  objects[i++] = light;
  objects[i++] = create_sphere (vec (45, 75, -40), 5);

  rings = create_ring (vec (60, 60, -30),
		       vec (2, 4, -1),
		       25, 20);
  objects[i++] = rings;
  objects[i++] = create_ring (vec (0, -35, 0.0),
			      vec (0, 1, 0),
			      100, 60);
  
  objects[i] = 0;

  render (options.width, options.height, options.rays_per_pixel);
  return 0;
}

Real
trace (Ray ray, int level)
{
  static Real random_normalize = 1.0 / RAND_MAX;
  Object *object;
  Real alpha;
  Vector normal;
  Vector specular;
  Vector diffuse;
  Real A;
  Ray newray;

  if (level > 420)
    {
      fprintf (stderr, "420 recursive calls to trace ().\n");
      return 0.0;
    }

  find_first_intersection (ray, &object, &alpha);

  if (object == 0)
    return 0.0;
    //return 0.5 * ray.d.y + 0.5;

  if (object == light)
    return 100.0;

  if (object == rings)
    return 0.2;

//  return 1.0;

  newray.p.x = ray.p.x + 0.99 * alpha * ray.d.x;
  newray.p.y = ray.p.y + 0.99 * alpha * ray.d.y;
  newray.p.z = ray.p.z + 0.99 * alpha * ray.d.z;

  object->normal (object, &newray.p, &normal);

  A = 2.0 * vector_scalar (normal, ray.d);
  specular.x = ray.d.x - A * normal.x;
  specular.y = ray.d.y - A * normal.y;
  specular.z = ray.d.z - A * normal.z;

  diffuse.x = random_normalize * random () - 0.5;
  diffuse.y = random_normalize * random () - 0.5;
  diffuse.z = random_normalize * random () - 0.5;
  if (vector_scalar (diffuse, normal) < 0.0)
    {
      diffuse.x = -diffuse.x;
      diffuse.y = -diffuse.y;
      diffuse.z = -diffuse.z;
    }
  vector_normalize (diffuse);

  if (1)
    newray.d = diffuse;
  else
    newray.d = specular;

  return trace (newray, level + 1);
}

inline void
find_first_intersection (Ray ray, 
			 Object **first_object,
			 Real *first_alpha)
{
  int i;
  Real alpha;
  
  *first_object = 0;
  *first_alpha = REAL_MAX;

  for (i = 0; objects[i] != 0; i++)
    {
      if ((alpha = objects[i]->intersection (objects[i], &ray)) != 0.0 &&
	  alpha > 0.0 && alpha < *first_alpha)
	{
	  *first_alpha = alpha;
	  *first_object = objects[i];
	}
    }
}

void
parse_options (int argc, char *argv[], struct Options *options)
{ 
  int c;

  /* Defaults. */
  options->N = 0;
  options->width = 70;
  options->height = 40;
  options->rays_per_pixel = 8;
  
  while ((c = getopt (argc, argv, "N:w:h:r:")) != EOF)
    switch (c)
      {
      case 'N': options->N = atoi (optarg); break;
      case 'w': options->width = atoi (optarg); break;
      case 'h': options->height = atoi (optarg); break;
      case 'r': options->rays_per_pixel = atoi (optarg); break;
      }
}
