/*

ray.h

(C) 1997 lars brinkhoff

*/

#include <math.h>

typedef double Real;
#define REAL_MAX 1e99

typedef struct
{
  Real x, y, z;
} Vector;

typedef struct
{
  Vector p, d;
} Ray;

#define vec(x, y, z) ((Vector){ x, y, z})  

static inline Real
vector_scalar (Vector u, Vector v)
{
  return u.x * v.x + u.y * v.y + u.z * v.z;
}

static inline Real
vector_magnitude (Vector v)
{
  return (Real)sqrt (vector_scalar (v, v));
}

#define vector_normalize(v) \
({ \
  Real magnitude = vector_magnitude (v); \
  Real c; \
  \
  if (magnitude == 0.0) \
    fprintf (stderr, "Normalizing zero vector!\n"); \
  else \
    { \
      c = 1 / magnitude; \
      (v).x *= c; \
      (v).y *= c; \
      (v).z *= c; \
    } \
})

typedef struct
{
  Real (*intersection)(void *, Ray *);
  void (*normal)(void *, Vector *, Vector *);
} Object;
