/*

ring.c

(C) 1997 lars brinkhoff

*/

#include <stdio.h>
#include <stdlib.h>

#include "ray.h"

struct Ring
{
  Object o;
  Vector p, n;
  Real rmax2, rmin2;
};

Real
ring_intersection (void *ring_, Ray *ray)
{
  struct Ring *ring = ring_;
  Real alpha;
  Real A, q2;
  Vector v;
  Vector q;

  A = vector_scalar (ray->d, ring->n);
  if (A == 0.0)
    return 0.0;
  
  v.x = ring->p.x - ray->p.x;
  v.y = ring->p.y - ray->p.y;
  v.z = ring->p.z - ray->p.z;
  alpha = vector_scalar (v, ring->n) / A;
  
  q.x = alpha * ray->d.x - v.x;
  q.y = alpha * ray->d.y - v.y;
  q.z = alpha * ray->d.z - v.z;

  q2 = vector_scalar (q, q);
  if (ring->rmin2 <= q2 && q2 <= ring->rmax2)
    return alpha;

  return 0.0;
}

void
ring_normal (void *ring_, Vector *p, Vector *n)
{
  struct Ring *ring = ring_;
  *n = ring->n;
}

Object *
create_ring (Vector p, Vector n, Real r1, Real r2)
{
  struct Ring *ring = malloc (sizeof (struct Ring));
  
  if (ring == 0)
    fprintf (stderr, "Out of memory!\n");
  else
    {
      ring->o.intersection = ring_intersection;
      ring->o.normal = ring_normal;
      ring->p = p;
      ring->n = n; vector_normalize (ring->n);
      ring->rmax2 = r1 * r1;
      ring->rmin2 = r2 * r2;
    }
  
  return (Object *)ring;
}
