/*

sphere.c

(C) 1997 lars brinkhoff

*/

#include <stdio.h>
#include <stdlib.h>

#include "ray.h"

struct Sphere
{
  Object o;
  Real x, y, z, r;
};


Real sphere_intersection (void *, Ray *);
void sphere_normal (void *, Vector *, Vector *);

Object *
create_sphere (Real x, Real y, Real z, Real r)
{
  struct Sphere *sphere = malloc (sizeof (struct Sphere));
  
  if (sphere == 0)
    fprintf (stderr, "Out of memory!\n");
  else
    {
      sphere->o.intersection = sphere_intersection;
      sphere->o.normal = sphere_normal;
      sphere->x = x;
      sphere->y = y;
      sphere->z = z;
      sphere->r = r;
    }
  
  return (Object *)sphere;
}

Real
sphere_intersection (void *sphere_, Ray *ray)
{
  struct Sphere *sphere = sphere_;
  Real D, E, F, G;
  Real alpha;
  Vector v;

  v.x = ray->p.x - sphere->x;
  v.y = ray->p.y - sphere->y;
  v.z = ray->p.z - sphere->z;
  D = vector_scalar (v, ray->d);
  E = vector_scalar (v, v) - sphere->r * sphere->r;
  F = D * D - E;
  if (F < 0.0)
    return 0.0;
  G = sqrt (F);
  alpha = -D - G;
  if (alpha > 0.0)
    return alpha;
  alpha = -D + G;
  if (alpha > 0.0)
    return alpha;
  return 0.0;
}

void
sphere_normal (void *sphere_, Vector *p, Vector *n)
{
  struct Sphere *sphere = sphere_;
  n->x = p->x - sphere->x;
  n->y = p->y - sphere->y;
  n->z = p->z - sphere->z;
  vector_normalize (*n);
}
