/*

pix.c

(C) 1997 lars brinkhoff

*/

#include <stdio.h>
#include <stdlib.h>

struct Options
{
  int ascii, raw, width, height;
};

void read_raw (struct Options *);
void read_ascii (struct Options *);
void pixel (int, int, double, double, double);
void parse_options (int, char *[], struct Options *);

extern char *optarg;
extern int optind, opterr, optopt;

int
main (int argc, char *argv[])
{
  struct Options options;

  parse_options (argc, argv, &options);

  if (options.raw)
    read_raw (&options);
  else if (options.ascii)
    read_ascii (&options);

  return 0;
}

void
read_raw (struct Options *options)
{
  int x, y;
  int r, g, b;

  for (y = 0; options->height > 0 ? y < options->height : 1; y++)
    for (x = 0; x < options->width; x++)
      {
	r = getchar();
	g = getchar();
	b = getchar();
	if (feof (stdin))
	  goto end;
	pixel (x, y, r / 255.0, g / 255.0, b / 255.0);
      }
end:
}

void
read_ascii (struct Options *options)
{
  int x, y;
  int r, g, b;
  char s[4];

  for (y = 0; ; y++)
    for (x = 0; ; x++)
      {
	s[0] = getchar();
	s[1] = getchar();
	s[2] = getchar();
	s[3] = 0;
	r = atoi (s);
	s[0] = getchar();
	s[1] = getchar();
	s[2] = getchar();
	s[3] = 0;
	g = atoi (s);
	s[0] = getchar();
	s[1] = getchar();
	s[2] = getchar();
	s[3] = 0;
	b = atoi (s);
	if (feof (stdin))
	  goto end;
	pixel (x, y, r / 999.0, g / 999.0, b / 999.0);
      }
end:
}

void
pixel (int x, int y, double r, double g, double b)
{
  /*
      The more inconsistent shapes are:  ~ ^ * & | ' [ ] < > 0 l y
      Characters that may not appear in a font are:  ^ ` # | { } ~ \ [ ] $ @
      Example:
        $@B%8&WM#*oahkbdpqwmZO0QLCJUYXzcvunxrjft/\|()1{}[]?-_+~<>i!lI;:,"^`'.
  */
//static char c[] = " .x%$#";
  static char c[] = " ,,//xX%O8B";
//static char c[] = "B8O%Xx/, ";
  static int last_y = 0;
  double i;
  int j;

  if (y != last_y)
    {
      last_y = y;
      putchar ('\n');
    }

  i = 0.3 * r + 0.6 * g + 0.1 * b;
  j = (int)((sizeof (c) - 2) * i + .5);

  putchar (c[j]);
}

void
parse_options (int argc, char *argv[], struct Options *options)
{ 
  int c;

  /* Defaults. */
  options->raw = 1;
  options->width = -1;
  options->height = -1;
  
  while ((c = getopt (argc, argv, "arw:h:?")) != EOF)
    switch (c)
      {
      case 'a': options->ascii = 1; options->raw = 0; break;
      case 'r': options->raw = 1; options->ascii = 0; break;
      case 'w': options->width = atoi (optarg); break;
      case 'h': options->height = atoi (optarg); break;
      case '?':
	fprintf (stderr,
"usage: %s -a -r -? -w width -h height\n"
"             -a          ASCII input.\n"
"             -r          Raw input.\n"
"             -?          Help.\n"
"             -w width    Sprcify input width.\n"
"             -h height   Specify input height.\n", argv[0]);
      }

  if (options->raw)
    {
      if (options->width < 0)
	{
	  fprintf (stderr,
		   "%s: raw input width must be specified.\n", argv[0]);
	  exit (1);
	}
    }
}
