/*

filter.c

(C) 1997 lars brinkhoff

*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

struct Matrix
{
  int x[5][5];
};

struct Options
{
  char *me;
  int width, height;
  int A[5][5];
};

extern char *optarg;
extern int optind, opterr, optopt;

int xgetchar ()
{
  int c;
  
  c = getchar ();

  if (c == EOF)
    return 0;
  else
    return c;
}

void
pixel (unsigned char *rbuf, unsigned char *gbuf, unsigned char *bbuf, int bufsize,
       int p, struct Options *options)
{
  double r = 0, g = 0, b = 0;
  double weight = 0;
  int i, j;
  int x, y;

  if (p < 0)
    return;

  x = p % options->width;
  y = p / options->width;

  for (i = -2; i < 3; i++)
    for (j = -2; j < 3; j++)
	if (x + i >= 0 && y + j >= 0 && x + i < options->width && y + j < options->height)
	  {
	    int bufpos = (options->width * (y + j) + x + i) % bufsize;
	    r += options->A[i + 2][j + 2] * rbuf[bufpos];
	    g += options->A[i + 2][j + 2] * gbuf[bufpos];
	    b += options->A[i + 2][j + 2] * bbuf[bufpos];
	    weight += options->A[i + 2][j + 2];
	  }

  putchar ((int)(r / weight + 0.5));
  putchar ((int)(g / weight + 0.5));
  putchar ((int)(b / weight + 0.5));
}

void do_filter (struct Options *options)
{
  int bufsize = 5 * options->width;
  unsigned char *rbuf = malloc (bufsize);
  unsigned char *gbuf = malloc (bufsize);
  unsigned char *bbuf = malloc (bufsize);
  int i, j;
  
  if (rbuf == 0 || gbuf == 0 || bbuf == 0)
    {
      fprintf (stderr, "%s: out of memory\n", options->me);
      exit (1);
    }

  memset (rbuf, 0, bufsize);
  memset (gbuf, 0, bufsize);
  memset (bbuf, 0, bufsize);
  
  while (!(feof (stdin) || i / options->width == options->height))
    {
      rbuf[i % bufsize] = xgetchar ();
      gbuf[i % bufsize] = xgetchar ();
      bbuf[i % bufsize] = xgetchar ();
      
      pixel (rbuf, gbuf, bbuf, bufsize,
	     i - 2 - 2 * options->width, options);
      
      i++;
    }

  options->height = i / options->width;

  for (j = 0; j < 2 + 2 * options->width; j++, i++)
    pixel (rbuf, gbuf, bbuf, bufsize,
	   i - 2 - 2 * options->width, options);
}

void
parse_options (int argc, char *argv[], struct Options *options)
{
  int A1[5][5] = { { 1, 2, 2, 2, 1 },
		   { 2, 4, 4, 4, 2 },
		   { 2, 4, 4, 4, 2 },
		   { 2, 4, 4, 4, 2 },
		   { 1, 2, 2, 2, 1 } };
  int c;

  options->me = argv[0];

  /* Defaults. */
  options->width = -1;
  options->height = 1000000;
  memcpy (&options->A, A1, sizeof (A1));

  while ((c = getopt (argc, argv, "arw:h:?")) != EOF)
    switch (c)
      {
      case 'w': options->width = atoi (optarg); break;
      case 'h': options->height = atoi (optarg); break;
      }

  if (options->width < 0)
    {
      fprintf (stderr,
	       "%s: input width must be specified.\n", argv[0]);
      exit (1);
    }
}

int
main (int argc, char *argv[])
{
  struct Options options;

  parse_options (argc, argv, &options);

  do_filter (&options);

  return 0;
}
